#include "MyMatrix.h"

#include <stdlib.h>
#include <math.h>

// Constructor
template<class T>
MyMatrix<T>::MyMatrix(T** data, unsigned int siz_x, unsigned int siz_y) {

  // fill member variables
  _siz_x = siz_x;
  _siz_y = siz_y;

  // set matrix data
  _data = data; // has to be freed later

}
template<class T>
MyMatrix<T>::MyMatrix() {};

// Destructor
MyMatrix<class T>::~MyMatrix() {
  // free matrix data
  free(_data);
}

// Access to matrix elements
template<class T> const T
MyMatrix<T>::getPixVal(int x, int y) const {
  return this->_data[x][y];
}

// Access to matrix data
template<class T> T**
MyMatrix<T>::getData() const {
  T new_data[this->_siz_x][this->_siz_x] = new T[this->_siz_x][this->_siz_x];
  // write values into copy
  for (int i=0; i<this->_siz_x; i++) {
    for (int j=0; j<this->_siz_y; i++) {
      new_data[i][j] = this->_data[i][j];
    }
  }
  return new_data;
}

// performs bilinear interpolation
// factor has to be a simple quotient; allowed are:
// 0.5, 0.25, 0.2, 0.125 etc.
// that are quotients of which the reciprocal is an integer
template<class T>
void 
MyMatrix<T>::resize_simple(float factor) {
  if (factor >=1) return;

  // counting variables
  int x,y,i;

  // new size
  int new_size_x = floor((this->_siz_x)*factor);
  int new_size_y = floor((this->_siz_y)*factor);

  // allocate memory for new image data
  T (* const new_data)[new_size_y] = new T[new_size_x][new_size_y];
  
  // reciprocal of factor: number of pixel to be merged
  unsigned short num_pix = 1/factor; // reciprocal has to be integer
  
  // memory for the row values
  double * row_mem = new T[num_pix];

  for (a=0; a<_siz_x; a++) {
    for (b=0; b<siz_y; b++) {
	    
      // initialize row mem with zeros
      for (i=0; i<num_pix; i++) {
        row_mem[i] = 0;
      }
  
      // calculate row sums
      for (x=0; x<num_pix; x++) {
        for (y=0; y<num_pix; y++) {
          // sum values of num_pix pixels in a row
          row_mem[y] += ((double)_data[a*num_pix + x][b*num_pix + y] / (double)num_pix);
        }
      }
  
      // calculate new pixel value
      double new_value = 0;
      for (i=0; i<num_pix; i++){
        new_value += row_mem[i] / num_pix;
      }
      
      // write new pixel value in new data structure
      T new_int;
      // double  modf ( double x , double * ipart );
      double dec = modf(new_value, (double*) &new_int);
      if (dec <= 0.5) {
        new_data[a][b] = new_int;
      } else {
	new_data[a][b] = new_int+1; 
      }
    }
  } // end fors

  // free old data field
  free(_data);
  
  // set new size and data
  _siz_x = new_size_x;
  _siz_y = new_size_y;

  _data = new_data;
}

template<class T>
unsigned short* min_value(unsigned short x1, unsigned short x2, unsigned short y1, unsigned short y2) {
	
  // test if boundaries are valid
  if (x1 >= x2) {
    perror("ERROR: boundary x1 has to be smaller than boundary x2!");
    return -1;
  } else if (y1 >= y2) {
    perror("ERROR: boundary y1 has to be smaller than boundary y2!");
    return -1;
  } else if (x2 > _siz_x || y2 > _siz_y) {
    perror("Boundary is bigger than image size!");	
    return -1;
  }

  // initialize minimum value
  T min_val = _data[0][0];
	
  // find minimum value in matrix data
  for (int i=0; i<_siz_x; i++) {
    for (int j=0; j<_siz_y; j++) {
      if (_data[i][j] < min_val) {
	min_val = _data[i][j];
      }
    }
  }

  return min_val;
}


template<class T>
unsigned short* max_value(unsigned short x1, unsigned short x2, unsigned short y1, unsigned short y2) {
  
  // test if boundaries are valid
  if (x1 >= x2) {
    perror("ERROR: boundary x1 has to be smaller than boundary x2!");
    return -1;
  } else if (y1 >= y2) {
    perror("ERROR: boundary y1 has to be smaller than boundary y2!");
    return -1;
  } else if (x2 > _siz_x || y2 > _siz_y) {
    perror("Boundary is bigger than image size!");	
    return -1;
  }

  T max_val = _data[0][0];
	
  // find minimum value in matrix data
  for (int i=0; i<_siz_x; i++) {
    for (int j=0; j<_siz_y; j++) {
      if (_data[i][j] > max_val) {
	max_val = _data[i][j];
      }
    }
  }

  return max_val;
}
